//------------------------------------------------------------
#include "RODOS5_6.h"
#include <iostream>
#include <string>
#include <sstream>
#include <iomanip>
//------------------------------------------------------------
//#define DEBUG
//------------------------------------------------------------
unsigned char RODOS56_t::CRC8(unsigned char CRC, unsigned char D)
{   //  подчсёт CRC для DALLAS
    unsigned char R=CRC;
    for (int i=0; i<8; i++)
        if (((R^(D>>i))&0x01)==0x01) R=((R^0x18)>>1)|0x80;
            else R=(R>>1)&0x7F;
    return R;
}
//------------------------------------------------------------
bool RODOS56_t::READ_ROM(unsigned long long &ROM64)
{   //  чтение ROM, 14ms
    bool RESULT=false;
    unsigned long B;
    unsigned char N=3;
    unsigned long long T, CRC;
    while (!RESULT&((N--)>0))
        if (OW_RESET())
            if (OW_WRITE_BYTE(0x33))
                {   //  чтение 64 бит
                ROM64=0;    CRC=0;
                if (OW_READ_4BYTE(B))
                    {
                    T=B;
                    ROM64=ROM64+T;
                    if (OW_READ_4BYTE(B)) { T=B; ROM64=ROM64+(T<<32); RESULT=true; }
                        else RESULT=false;
                    }
                    else RESULT=false;
                //   проверка CRC
                if (RESULT)
                    {
                    T=ROM64;
                    for (int i=0; i<8; i++) CRC=CRC8(CRC, (T>>(i*8))&0xFF);
                    RESULT=CRC==0;
                    }
                }
#ifdef DEBUG
    if (!RESULT) std::cout << "Ошибка READ_ROM" << std::endl;
#endif
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::MATCH_ROM(unsigned long long ROM)
{   //  выбор прибора по ROM, 14ms
    bool RESULT=false;
    unsigned long long T=ROM;
    unsigned char N=3;
    while (!RESULT&((N--)>0))
        if (OW_RESET())
            if (OW_WRITE_BYTE(0x55))
                if (OW_WRITE_4BYTE(T&0xFFFFFFFF))
                    RESULT=OW_WRITE_4BYTE((T>>32)&0xFFFFFFFF);
#ifdef DEBUG
    if (!RESULT) std::cout << "Ошибка MATCH_ROM" << std::endl;
#endif
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::SKIP_ROM()
{   //  пропуск ROM-команд, 6ms
    bool RESULT=false;
    unsigned char N=3;
    while (!RESULT&((N--)>0))
        if (OW_RESET()) RESULT=OW_WRITE_BYTE(0xCC);
#ifdef DEBUG
    if (!RESULT) std::cout << "Ошибка SKIP_ROM" << std::endl;
#endif
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::SEARCH_ROM(unsigned long long ROM_NEXT, int PL)
{   //  поиск ROM, 1 dev - 410ms, 5 dev - 2.26s, 20 dev - 8.89s
    bool RESULT=false;
    unsigned char N=3;
    unsigned char BIT;
    bool CL[64]; for (int i=0; i<64; i++) CL[i]=false;
    unsigned long long RL[64];
    unsigned long long B1=1, CRC, ROM;
    while (!RESULT&((N--)>0))
        {
        ROM=0;
        if (OW_RESET()) RESULT=OW_WRITE_BYTE(0xF0);
        if (RESULT) {
            for (int i=0; i<64; i++) {
                if (RESULT) {
                    if (OW_READ_2BIT(BIT)) {
                        switch (BIT&0x03)
                            {
                            case 0 :
                                {   //  коллизия есть
                                if (PL<i) {CL[i]=true; RL[i]=ROM;}
                                if (PL>=i) BIT=(ROM_NEXT>>i)&0x01; else BIT=0;
                                if(!OW_WRITE_BIT(BIT)) { RESULT=false; i=64; }
                                if (BIT==1) ROM=ROM+(B1<<i);
                                break;
                                }
                            case 1 : { if (!OW_WRITE_BIT(0x01)) { RESULT=false; i=64; } else ROM=ROM+(B1<<i); break;}
                            case 2 : { if (!OW_WRITE_BIT(0x00)) { RESULT=false; i=64; } break;}
                            case 3 : { RESULT=false; i=64; break;}   //  нет на линии
                            }
                    }
                    else { RESULT=false; i=64; }
                }
            }
        }
        if (ROM==0) RESULT=false;
        if (RESULT) { CRC=0; for (int j=0; j<8; j++) CRC=CRC8(CRC, (ROM>>(j*8))&0xFF); RESULT=CRC==0; }
        }
    if (!RESULT)
    {
#ifdef DEBUG
    std::cout << "Ошибка SEARCH_ROM" << std::endl;
#endif
    }
        else ONEWIRE_ROM[ONEWIRE_COUNT++]=ROM;
    //  рекурентный вызов поиска
    for (int i=0; i<64; i++)
        if (CL[i]) SEARCH_ROM(RL[i]|(B1<<i), i);
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::SKIP_ROM_CONVERT()
{   //  пропуск ROM-команд, старт измерения температуры, 9ms
    bool RESULT=false;
    unsigned char N=3;
    while (!RESULT&((N--)>0))
        if (OW_RESET())
            if (OW_WRITE_BYTE(0xCC))
                RESULT=OW_WRITE_BYTE(0x44);
#ifdef DEBUG
    if (!RESULT) std::cout << "Ошибка SKIP_ROM_CONVERT" << std::endl;
#endif
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::GET_TEMPERATURE(unsigned long long ROM, float &T)
{   //  чтение температуры, 28ms
    unsigned long long CRC;
    unsigned long L1, L2;
    unsigned char L3;
    unsigned char FAMILY=ROM&0xFF;
    bool RESULT=false;
    unsigned char N=3;
    while ((!RESULT)&&((N--)>0))
        if (MATCH_ROM(ROM))
            if (OW_WRITE_BYTE(0xBE))
                    if (OW_READ_4BYTE(L1))
                        if (OW_READ_4BYTE(L2))
                            if (OW_READ_BYTE(L3))
                                {
                                CRC=0;
                                for (int i=0; i<4; i++) CRC=CRC8(CRC, (L1>>(i*8))&0xFF);
                                for (int i=0; i<4; i++) CRC=CRC8(CRC, (L2>>(i*8))&0xFF);
                                CRC=CRC8(CRC, L3);
                                RESULT=CRC==0;
                                short K=L1&0xFFFF;
                                //  DS18B20 +10.125=00A2h, -10.125=FF5Eh
                                //  DS18S20 +25.0=0032h, -25.0=FFCEh
                                //  K=0x0032;
                                T=1000;     //  для неопознанной FAMILY датчик отсутствует
                                if ((FAMILY==0x28)|(FAMILY==0x22)) T=K*0.0625;  //  DS18B20 | DS1822
                                if (FAMILY==0x10) T=K*0.5;                      //  DS18S20 | DS1820
                                }
#ifdef DEBUG
    if (!RESULT) std::cout << "Ошибка GET_TEMPERATURE" << std::endl;
#endif
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::READ_TEMPERATURE(void)
{
    bool RESULT = true;

    SEARCH_ROM(0, 0);
    if (ONEWIRE_COUNT <= 0) { std::cout << "Датчики не найдены" << std::endl; RESULT = false; }
    else
    {
        if (!SKIP_ROM_CONVERT()) { std::cout << "Ошибка SKIP_ROM_CONVERT" << std::endl; RESULT = false; }
        else
        {
             std::cout << "Измерение температуры..." << std::endl;
             usb_t::Sleep(1000);
             std::cout << "Измерение температуры завершено" << std::endl;

             float T;
             for (int i=0; i<ONEWIRE_COUNT; i++)
             if (GET_TEMPERATURE(ONEWIRE_ROM[i], T))
				{
                 std::cout << "ROM=" << std::setfill('0') << std::setw(16) << std::hex << ONEWIRE_ROM[i] << " T=" << std::fixed << std::setprecision(4) << T << std::endl;
                }
        }
    }
    return RESULT;
}
//------------------------------------------------------------
void RODOS56_t::PORTS(int COMMAND)
{   //  PORTB3 - 0x08
    //  PORTB4 - 0x10
    unsigned char PS;
    if (USB_GET_PORT(PS) == false) { std::cout << "Ошибка USB_GET_PORT" << std::endl; return; }
    //  включение / выключение
    if (COMMAND==PORT1ON)  { PS=PS|0x08; USB_SET_PORT(PS); std::cout << "Выход 1 установлен" << std::endl; }
    if (COMMAND==PORT1OFF) { PS=PS&0x10; USB_SET_PORT(PS); std::cout << "Выход 1 установлен" << std::endl; }
    if (COMMAND==PORT2ON)  { PS=PS|0x10; USB_SET_PORT(PS); std::cout << "Выход 2 установлен" << std::endl; }
    if (COMMAND==PORT2OFF) { PS=PS&0x08; USB_SET_PORT(PS); std::cout << "Выход 2 установлен" << std::endl; }
    //  запись состояния при включении
    if (COMMAND==PORTSSET)
        {
        if (USB_EE_WR(0x04, PS)) std::cout << "Состояние выходов при включении записано" << std::endl;
            else std::cout << "Ошибка записи состояние выходов при включении" << std::endl;
        }
}
//------------------------------------------------------------
void RODOS56_t::TERMOSTATE_READ()
{   //  чтение границ температуры из USB
    unsigned char H,L; short S;
    if (USB_EE_RD(0x10, H)&&USB_EE_RD(0x11, L)) { S=(H<<8)+L; std::cout << "T1H: " << S/16.0 << std::endl; }
        else std::cout << "T1H: ERR" << std::endl;
    if (USB_EE_RD(0x12, H)&&USB_EE_RD(0x13, L)) { S=(H<<8)+L; std::cout << "T1L: " << S/16.0 << std::endl; }
        else std::cout << "T1L: ERR" << std::endl;
    if (USB_EE_RD(0x14, H)&&USB_EE_RD(0x15, L)) { S=(H<<8)+L; std::cout << "T2H: " << S/16.0 << std::endl; }
        else std::cout << "T2H: ERR" << std::endl;
    if (USB_EE_RD(0x16, H)&&USB_EE_RD(0x17, L)) { S=(H<<8)+L; std::cout << "T2L: " << S/16.0 << std::endl; }
        else std::cout << "T2L: ERR" << std::endl;
}

//------------------------------------------------------------
bool RODOS56_t::TERMOSTATE_WRITE(int TS, float Ftemp)
{   //  запись границ температуры в USB
    short Stemp = Ftemp*16;
    //  запись в USB
    bool RESULT=true;

    switch (TS)
    {
    case T1H: { RESULT=RESULT&USB_EE_WR(0x10, Stemp>>8); RESULT=RESULT&USB_EE_WR(0x11, Stemp&0xFF); break; }
    case T1L: { RESULT=RESULT&USB_EE_WR(0x12, Stemp>>8); RESULT=RESULT&USB_EE_WR(0x13, Stemp&0xFF); break; }
    case T2H: { RESULT=RESULT&USB_EE_WR(0x14, Stemp>>8); RESULT=RESULT&USB_EE_WR(0x15, Stemp&0xFF); break; }
    case T2L: { RESULT=RESULT&USB_EE_WR(0x16, Stemp>>8); RESULT=RESULT&USB_EE_WR(0x17, Stemp&0xFF); break; }
    default: { RESULT = false; break; }
    }
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::TERMOSTATE_SET(unsigned long long ROM)
{
    bool RESULT=true;
    bool ROM_verified = false;

	if ( ONEWIRE_COUNT <= 0 ) { SEARCH_ROM(0, 0); }

    for (int i=0; i<ONEWIRE_COUNT; i++)  { if (ROM == ONEWIRE_ROM[i]) ROM_verified = true; }
    if (ROM_verified)
    {
        for (int i=0; i<8; i++)
            RESULT=RESULT&USB_EE_WR(0x08+i, (ROM>>(i*8))&0xFF);
    }
    else RESULT = false;
    return RESULT;
}
//------------------------------------------------------------
bool RODOS56_t::AUTONOM_DELAY_SET(unsigned char DELAY)
{
 return USB_EE_WR(0x05, DELAY);
}
//------------------------------------------------------------
signed int RODOS56_t::AUTONOM_DELAY_READ(void)
{
 unsigned char RecieveData;
 if ( USB_EE_RD(0x05, RecieveData) == false ) { return (-1); }
 else { return RecieveData; }
}
//------------------------------------------------------------

